'use strict'
var __importDefault =
    (this && this.__importDefault) ||
    function (mod) {
        return mod && mod.__esModule ? mod : { default: mod }
    }
Object.defineProperty(exports, '__esModule', { value: true })
exports.logger = exports.loggerSetup = void 0
const debug_1 = __importDefault(require('debug'))
const helpers = require('./helpers')
let LOGGER_DEBUG_COLOR = process.env.LOGGER_DEBUG_COLOR || '4'
let LOGGER_ERROR_COLOR = process.env.LOGGER_ERROR_COLOR || '9'
let LOGGER_TRACE_COLOR = process.env.LOGGER_TRACE_COLOR || '15'
let LOGGER_DEBUG_ENABLED =
    (process.env.LOGGER_DEBUG_ENABLED &&
        process.env.LOGGER_DEBUG_ENABLED === 'true') ||
    false
let LOGGER_ERROR_ENABLED =
    (process.env.LOGGER_ERROR_ENABLED &&
        process.env.LOGGER_ERROR_ENABLED === 'true') ||
    true
let LOGGER_TRACE_ENABLED =
    (process.env.LOGGER_TRACE_ENABLED &&
        process.env.LOGGER_TRACE_ENABLED === 'true') ||
    false
let LOGGER_TIMESTAMP_ENABLED = process.env.LOGGER_TIMESTAMP_ENABLED || false
const loggerSetup = ({
    debugColor,
    debugEnabled,
    errorColor,
    errorEnabled,
    traceColor,
    traceEnabled,
    timestampEnabled,
}) => {
    if (debugColor !== undefined) {
        LOGGER_DEBUG_COLOR = debugColor
    }
    if (debugEnabled !== undefined) {
        LOGGER_DEBUG_ENABLED = debugEnabled
    }
    if (errorColor !== undefined) {
        LOGGER_ERROR_COLOR = errorColor
    }
    if (errorEnabled !== undefined) {
        LOGGER_ERROR_ENABLED = errorEnabled
    }
    if (traceColor !== undefined) {
        LOGGER_TRACE_COLOR = traceColor
    }
    if (traceEnabled !== undefined) {
        LOGGER_TRACE_ENABLED = traceEnabled
    }
    if (timestampEnabled !== undefined) {
        if (typeof timestampEnabled === 'boolean') {
            LOGGER_TIMESTAMP_ENABLED = timestampEnabled
        } else {
            timestampEnabled.split(',').forEach((n) => {
                helpers.setTimestamp(debug_1.default, true, n)
            })
        }
    }
}
exports.loggerSetup = loggerSetup
const logMessage = (callback, prefix, node) => {
    return (message) => {
        const messagePrefix = node ? `${node.name}:${node.id}` : prefix
        if (messagePrefix) {
            return callback(`[${messagePrefix}] ${message}`)
        } else {
            return callback(`${message}`)
        }
    }
}
const logger = (namespacePrefix, namespace, messagePrefix, node) => {
    helpers.setTimestamp(
        debug_1.default,
        LOGGER_TIMESTAMP_ENABLED,
        namespacePrefix
    )
    const debug = debug_1.default(
        namespace ? `${namespacePrefix}:${namespace}` : namespacePrefix
    )
    debug.color = LOGGER_DEBUG_COLOR
    if (LOGGER_DEBUG_ENABLED) {
        debug.enabled = LOGGER_DEBUG_ENABLED
    }
    const logDebug = logMessage(debug, messagePrefix, node)
    const error = debug_1.default(
        namespace
            ? `${namespacePrefix}-Error:${namespace}`
            : `${namespacePrefix}-Error`
    )
    error.color = LOGGER_ERROR_COLOR
    error.enabled = LOGGER_ERROR_ENABLED
    const logError = (message, nodeError = true, injectedNode) => {
        const nodeToUse = injectedNode || node
        if (nodeToUse && nodeError) {
            nodeToUse.error(message)
        }
        logMessage(error, messagePrefix, nodeToUse)(message)
    }
    const trace = debug_1.default(
        namespace
            ? `${namespacePrefix}-Trace:${namespace}`
            : `${namespacePrefix}-Trace`
    )
    trace.color = LOGGER_TRACE_COLOR
    if (LOGGER_TRACE_ENABLED) {
        trace.enabled = LOGGER_TRACE_ENABLED
    }
    const logTrace = logMessage(trace, messagePrefix, node)
    return {
        debug: logDebug,
        error: logError,
        trace: logTrace,
    }
}
exports.logger = logger
module.exports = { logger: exports.logger, loggerSetup: exports.loggerSetup }
